--[[
	Copyright (C) 2006-2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local L = AceLibrary("AceLocale-2.2"):new("Manufac")
Manufac = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0")
Manufac:RegisterDB("ManufacDB")
local dewdrop = AceLibrary("Dewdrop-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local stitch = AceLibrary("Stitch-1.0")
--big scary locals:
--toggles mod standby
local enabled=false
--tracks which was opened more recently, in case both are open, so we know what we're dealing with.
local recentopen
--tracks spellbook ids so we can cast from dewdrop
local skills={}
--store the defaults for the professions we actually have
local loadeddefaults={}
--this is set if we're opening a profession using the fubar menu
local fubaropening=false
local professionpoints = {} --["Alchemy"] = 300
--!!DELETE ME
local timer
--stores profession color data
local colors = {
	["optimal"] = "|cFFFF8844",
	["medium"] = "|cFFFFFF00",
	["easy"] = "|cFF44CC44",
	["trivial"] = "|cFFAAAAAA"
}

--the slash commands
local options = {
	type = 'group',
	args = {
		reset = {
			type = 'execute',
			name = L["Reset"],
			desc = L["Reset the profession groups"],
			func = function()
				Manufac:Reset()
				Manufac:Print(L["Reset the profession groups"])
			end,
			order = 501,
		},
		unsorted = {
			type = 'execute',
			name = L["Unsorted"],
			desc = L["Unsorted"],
			func = function()
				Manufac:ListUnsortedRecipes()
			end,
			order = 500,
		},
		sorttype = {
			type = 'text',
			name = L["Sort Type"],
			desc = L["Change between Difficulty and Alphabetical sorting"],
			get = function()
				return Manufac.db.profile.sorttype
			end,
			set = function(v)
				Manufac.db.profile.sorttype = v
				Manufac:Print(L["Sort Type"]..": ["..v.."]")
			end,
			validate = {L["Alpha"], L["Difficulty"]},
			order = 5,
		},
		money = {
			name = L["Money Summary"],
			type = 'toggle',
			desc = L["Toggle money summary"],
			get = function()
				return Manufac.db.profile.money
			end,
			set = function(v)
				Manufac.db.profile.money = v
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
			order = 6,
		},
		position = {
			type = 'text',
			name = L["Position"],
			desc = L["Change popup position behavior"],
			get = function()
				return Manufac.db.profile.position
			end,
			set = function(v)
				Manufac.db.profile.position = v
				Manufac:Position()
				Manufac:Print(L["Position"]..": ["..v.."]")
			end,
			validate = {L["Mouse"], L["Center"], L["Left"], L["Right"], L["Bottom"], L["BottomLeft"],
				L["BottomRight"], L["Top"], L["TopLeft"], L["TopRight"]},
			order = 7,
		},
		offsetx = {
			type = 'text',
			name = L["X Offset"],
			desc = L["Set X Offset"],
			usage = "<number>",
			get = function()
				return Manufac.db.profile.offsetx
			end,
			set = function(newx)
				Manufac.db.profile.offsetx = tonumber(newx)
				Manufac:Position()
				Manufac:Print(L["X Offset"]..": ["..newx.."]")
			end,
			validate = function(value)
				if tonumber(value) then
					return true
				else
					return false
				end
			end,
			order = 8,
		},
		offsety = {
			type = 'text',
			name = L["Y Offset"],
			desc = L["Set Y Offset"],
			usage = "<number>",
			get = function()
				return Manufac.db.profile.offsety
			end,
			set = function(newy)
				Manufac.db.profile.offsety = tonumber(newy)
				Manufac:Position()
				Manufac:Print(L["Y Offset"]..": ["..newy.."]")
			end,
			validate = function(value)
				if tonumber(value) then
					return true
				else
					return false
				end
			end,
			order = 9,
		},
		fubar = {
			name = L["Use FuBar"],
			type = 'toggle',
			desc = L["Use FuBar"],
			get = function()
				return Manufac.db.profile.usefubar
			end,
			set = function(v)
				Manufac.db.profile.usefubar = v
				Manufac:SetFubarPluginState()
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
			order = 40,
		},
		level2tooltips = {
			name = L["Toggle Level 2 Tooltips"],
			type = 'toggle',
			desc = L["Toggle Level 2 Tooltips"],
			get = function()
				return Manufac.db.profile.l2tooltips
			end,
			set = function(v)
				Manufac.db.profile.l2tooltips = v
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
			order = 12,
		},
		level3tooltips = {
			name = L["Toggle Level 3 Tooltips"],
			type = 'toggle',
			desc = L["Toggle Level 3 Tooltips"],
			get = function()
				return Manufac.db.profile.l3tooltips
			end,
			set = function(v)
				Manufac.db.profile.l3tooltips = v
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
			order = 12,
		},
		canmake = {
			name = L["Show Can Make"],
			type = 'toggle',
			desc = L["Show Can Make"],
			get = function()
				return Manufac.db.profile.canmake
			end,
			set = function(v)
				Manufac.db.profile.canmake = v
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
			order = 13,
		},
	}
}
function Manufac:OnInitialize()
	Manufac:RegisterChatCommand({"/manufac", "/manu"}, options)
	--Manufac:SetDebugging(true)
	Manufac:Debug("Initialized")
	CreateFrame("Frame", "ManufacFrame")
	ManufacFrame:SetFrameStrata("BACKGROUND")
	ManufacFrame:SetWidth(1)
	ManufacFrame:SetHeight(1)
	ManufacFrame:SetPoint("CENTER",0,0)
	ManufacFrame:Show()
	self:RegisterDefaults('char', {
		['*'] = { ['*'] = "Unsorted" },
	})
end
function Manufac:Reset()
	--Manufac:ResetDB()  --doesnt seem to properly reset the data for the current session,
	--stuff stays sorted if you just use a resetdb(), so then we use the cruder method..
	if not Manufac.db.profile.groups then Manufac.db.profile.groups = {} end
	if Manufac.db.profile.groups[BS["Alchemy"]] then Manufac.db.profile.groups[BS["Alchemy"]] = {} end
	if Manufac.db.profile.groups[BS["Engineering"]] then Manufac.db.profile.groups[BS["Engineering"]] = {} end
	if Manufac.db.profile.groups[BS["Enchanting"]] then Manufac.db.profile.groups[BS["Enchanting"]] = {} end
	if Manufac.db.profile.groups[BS["Leatherworking"]] then Manufac.db.profile.groups[BS["Leatherworking"]] = {} end
	if Manufac.db.profile.groups[BS["Tailoring"]] then Manufac.db.profile.groups[BS["Tailoring"]] = {} end
	if Manufac.db.profile.groups[BS["Blacksmithing"]] then Manufac.db.profile.groups[BS["Blacksmithing"]] = {} end
	if Manufac.db.profile.groups[BS["First Aid"]] then Manufac.db.profile.groups[BS["First Aid"]] = {} end
	if Manufac.db.profile.groups[BS["Cooking"]] then Manufac.db.profile.groups[BS["Cooking"]] = {} end
	if Manufac.db.profile.groups[BS["Jewelcrafting"]] then Manufac.db.profile.groups[BS["Jewelcrafting"]] = {} end
	if Manufac.db.profile.groups[BS["Smelting"]] then Manufac.db.profile.groups[BS["Smelting"]] = {} end
	if Manufac.db.profile.groups[BS["Poisons"]] then Manufac.db.profile.groups[BS["Poisons"]] = {} end
	Manufac.db.profile.offsetx = 0
	Manufac.db.profile.offsety = 0
	Manufac:Position()
	dewdrop:Close()
	--this is so the dewdrop doesnt try to redraw itself right this second (if it's open), we want it to re-assign the recipes to the "unsorted" category first
	skill="RESET"
end
function Manufac:OnEnable()
	enabled=true
	dewdrop:Register(ManufacFrame,
		'children', Manufac.menu
	)
	Manufac:ScanActionButtons()
	Manufac:RegisterEvent("TRADE_SKILL_SHOW")
	Manufac:RegisterEvent("CRAFT_SHOW")
	Manufac:RegisterEvent("SPELLS_CHANGED")
	Manufac:RegisterEvent("CRAFT_CLOSE")
	Manufac:RegisterEvent("TRADE_SKILL_CLOSE")
	Manufac:RegisterEvent("Manufac_FuBar_Open")
	if Manufac.db.profile.sorttype == nil then
		Manufac.db.profile.sorttype = L["Alpha"]
	end
	if Manufac.db.profile.position == nil then
		Manufac.db.profile.position = L["Mouse"]
	end
	if Manufac.db.profile.money == nil then
		Manufac.db.profile.money = true
	end
	if Manufac.db.profile.offsetx == nil then
		Manufac.db.profile.offsetx = 0
	end
	if Manufac.db.profile.offsety == nil then
		Manufac.db.profile.offsety = 0
	end
	if Manufac.db.profile.usefubar == nil then
		Manufac.db.profile.usefubar = true
	end
	if Manufac.db.profile.l2tooltips == nil then
		Manufac.db.profile.l2tooltips = true
	end
	if Manufac.db.profile.l3tooltips == nil then
		Manufac.db.profile.l3tooltips = true
	end
	if Manufac.db.profile.canmake == nil then
		Manufac.db.profile.canmake = true
	end
	if Manufac.db.profile.groups == nil then
		Manufac.db.profile.groups = {}
	end
	Manufac:SetFubarPluginState()
	Manufac:Position()
	Manufac:ModifyTradeSkill()
	stitch:EnableDataGathering("Manufac")
end
function Manufac:Position()
	ManufacFrame:ClearAllPoints()
	if Manufac.db.profile.position==L["Center"] then
		ManufacFrame:SetPoint("CENTER",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["Left"] then
		ManufacFrame:SetPoint("LEFT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["Right"] then
		ManufacFrame:SetPoint("RIGHT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["Bottom"] then
		ManufacFrame:SetPoint("BOTTOM",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["BottomLeft"] then
		ManufacFrame:SetPoint("BOTTOMLEFT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["BottomRight"] then
		ManufacFrame:SetPoint("BOTTOMRIGHT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["Top"] then
		ManufacFrame:SetPoint("TOP",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["TopLeft"] then
		ManufacFrame:SetPoint("TOPLEFT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	elseif Manufac.db.profile.position==L["TopRight"] then
		ManufacFrame:SetPoint("TOPRIGHT",Manufac.db.profile.offsetx,Manufac.db.profile.offsety)
	end
end
function Manufac:OnDisable()
	enabled=false
	Manufac:UnModifyTradeSkill()
	Manufac:UnModifyCraft()
	dewdrop:Close()
	dewdrop:Unregister(ManufacFrame)
	Manufac:UnregisterAllEvents()
	Manufac:SetFubarPluginState()
	stitch:DisableDataGathering("Manufac")
end
function Manufac:CheckDefaults(profession)
	if Manufac_Defaults and Manufac_Defaults[profession] then
		loadeddefaults[profession] = Manufac:FormatDefaultsTable(Manufac_Defaults[profession])
	end
end
function Manufac:FormatDefaultsTable(defaultstable)
	local resulttable = {}
	for k,v in pairs(defaultstable) do
		for word in string.gmatch(v, "(%d+);") do
			resulttable[word] = k
		end
	end
	return resulttable
end
function Manufac:SPELLS_CHANGED()
	Manufac:ScanActionButtons()
end
function Manufac:CRAFT_SHOW()
	UIParent.left = nil
	local centerFrame = GetCenterFrame()
	if ( centerFrame ) then
		local info = UIPanelWindows[centerFrame:GetName()]
		if ( info and (info.area == "left") ) then
			MovePanelToLeft()
			return
		end
	end

	recentopen="Craft"
	Manufac:GetSkillName()
	if skill == BS["Enchanting"] then Manufac:ScanTrade(skill) else Manufac:UnModifyCraft() end
	if enabled and fubaropening == false and (CraftFrameTitleText:GetText()==BS["Enchanting"]) then
		if Manufac.db.profile.position == L["Mouse"] then
			local x, y = GetCursorPosition()
			ManufacFrame:ClearAllPoints()
			ManufacFrame:SetPoint("BOTTOMLEFT",tonumber(x+Manufac.db.profile.offsetx),tonumber(y+Manufac.db.profile.offsety))
		end
		Manufac:ModifyCraft()
		dewdrop:Open(ManufacFrame)
	elseif fubaropening == true and CraftFrameTitleText:GetText()==BS["Enchanting"] then
		Manufac:ModifyCraft()
	end
	fubaropening=false
end
function Manufac:TRADE_SKILL_SHOW()
	UIParent.left = nil
	local centerFrame = GetCenterFrame()
	if ( centerFrame ) then
		local info = UIPanelWindows[centerFrame:GetName()]
		if ( info and (info.area == "left") ) then
			MovePanelToLeft()
			return
		end
	end

	recentopen="Trade"
	Manufac:GetSkillName()
	Manufac:ScanTrade(skill)
	if enabled and fubaropening == false then
		if Manufac.db.profile.position == L["Mouse"] then
			local x, y = GetCursorPosition()
			ManufacFrame:ClearAllPoints()
			ManufacFrame:SetPoint("BOTTOMLEFT",tonumber(x+Manufac.db.profile.offsetx),tonumber(y+Manufac.db.profile.offsety))
		end
		dewdrop:Open(ManufacFrame)
	end
	fubaropening=false
end
function Manufac:CRAFT_CLOSE()
	if dewdrop:IsOpen(ManufacFrame) then dewdrop:Close() end
	if CraftFrameTitleText then CraftFrameTitleText:SetText("") end
	if skill== BS["Enchanting"] then skill=nil end
	fubaropening=false
end
function Manufac:TRADE_SKILL_CLOSE()
	if dewdrop:IsOpen(ManufacFrame) then dewdrop:Close() end
	if TradeSkillFrameTitleText then TradeSkillFrameTitleText:SetText("") end
	if skill~= BS["Enchanting"] then skill=nil end
	fubaropening=false
end
function Manufac:TRADE_SKILL_UPDATE()
	if dewdrop:GetOpenedParent() then
		if dewdrop:IsOpen(ManufacFrame) then
			dewdrop:Refresh(3)
		elseif dewdrop:IsOpen(FuBarPluginManufacFrame) or dewdrop:IsOpen(FuBarPluginManufacFrameMinimapButton) then
			dewdrop:Refresh(4)
		end
	end
end
function Manufac:CRAFT_UPDATE()
	if dewdrop:GetOpenedParent() then
		if dewdrop:IsOpen(ManufacFrame) then
			dewdrop:Refresh(3)
		elseif dewdrop:IsOpen(FuBarPluginManufacFrame) or dewdrop:IsOpen(FuBarPluginManufacFrameMinimapButton) then
			dewdrop:Refresh(4)
		end
	end
end
function Manufac:FuDew(frame)
	dewdrop:Open(frame,
		'children', Manufac.substitutemenu
	)
end
function Manufac:RightClickMenu(arg1, arg2, arg3, arg4)
	Manufac:Debug(arg1, arg2, arg3, arg4)
	if arg1 == 1 then
		dewdrop:AddLine(
			'text', L["Options"],
			'value', L["Options"],
			'hasArrow', true
		)
	elseif arg1>1 and (arg2==L["Options"] or arg3==L["Options"] or arg4==L["Options"]) then
		dewdrop:FeedAceOptionsTable(options, 1)
	end
end
function Manufac:Manufac_FuBar_Open()
	ManufacFu:OnClick()
end
function Manufac:substitutemenu(arg1, arg2, arg3, arg4, arg5)
	Manufac:Debug("-----SUB MENU STARTED-----")
	if arg1==L["Options"] or arg2==L["Options"] or arg3==L["Options"] or arg4==L["Options"] then
		dewdrop:FeedAceOptionsTable(options, 1)
		return
	end
	if arg1==nil then
		for k,v in pairs(skills) do
			--make an option for each profession
			if k ~= BS["Disenchant"] and k ~= "level" then
				local l = v
				dewdrop:AddLine(
					'text', k,
					'value', k,
					'func', function()
						fubaropening=true
						CastSpell(l,"spell")
						Manufac:ScheduleEvent("Manufac_FuBar_Open", 0.15)
					end,
					'hasArrow', true
				)
			end
		end
		dewdrop:AddLine()
		--add the options here instead of on the same level as the categories
		dewdrop:AddLine(
			'text', L["Options"],
			'value', L["Options"],
			'hasArrow', true
		)
		--and a refresh button
		dewdrop:AddLine(
			'text', L["Refresh"],
			'func', Manufac.Refresh
		)
		dewdrop:AddLine(
			'text', L["Close"],
			'func', function()
				dewdrop:Close()
				HideUIPanel(CraftFrame)
				HideUIPanel(TradeSkillFrame)
			end
		)
		return
		--if we're higher up in the fubar menu then just change the args so that we end up in the right place.
	elseif arg1 and arg2==nil then
		Manufac:GetSkillName(arg1)
		if skill==arg1 then
			arg1=nil
		else
			dewdrop:AddLine(
				'text', L["Refresh Needed, Click Here"],
				'value', arg1,
				'func', function()
					fubaropening=true
					CastSpell(skills[arg1],"spell")
				end
			)
			return
		end
	elseif arg2 and arg3==nil then
		arg2=nil
	elseif arg3 and arg4==nil then
		arg3=nil
	elseif arg4 and arg5==nil then
		arg4=nil
	end
	Manufac:Debug("-----SUB MENU COMPLETE-----")
	Manufac:menu(arg1, arg2, arg3, arg4, arg5, true)
end
function Manufac:menu(arg1, arg2, arg3, arg4, arg5, arg6)
	timer = GetTime()
	if (not dewdrop:IsOpen()) then return end
	Manufac:Debug("-----MENU STARTED-----", arg1, arg2, arg3, arg4)
	if enabled==false then
		dewdrop:AddLine(
			'text', L["Manufac is Disabled, /manufac standby"],
			'isTitle', true
		)
		return
	end
	if skill=="RESET" then
		skill=nil
		dewdrop:Close()
		HideUIPanel(CraftFrame)
		HideUIPanel(TradeSkillFrame)
		return
	end
	if arg1==L["Options"] or arg2==L["Options"] or arg3==L["Options"] or arg4==L["Options"] then
		dewdrop:FeedAceOptionsTable(options, 1)
		return
	end
	if (skill == BS["Enchanting"]) then
		Manufac:ModifyCraft()
	end
	if TradeSkillFrame and enabled then
		Manufac:ModifyTradeSkill()
	end
	if arg1==nil and tostring(skill) then
		Manufac:Debug("-----STARTING FIRST LEVEL-----")
		--k, now start populating the menu.
		if tostring(professionpoints[skill]) then
			dewdrop:AddLine(
				'text', skill..professionpoints[skill],
				'isTitle', true
			)
		else
			dewdrop:AddLine(
				'text', L["Refresh Needed, Click Here"],
				'value', arg1,
				'func', function()
					fubaropening=true
					CastSpell(skills[skill],"spell")
				end
			)
			return
		end


		local result = {}
		local added = {}
		local haveresult=false
		for k,v in pairs(Manufac.db.profile.groups[skill]) do
			if not added[v] then
				table.insert(result, v)
			end
			added[v] = true
		end
		sort(result)
		dewdrop:AddLine()
		for k,v in pairs(result) do
			dewdrop:AddLine(
				'text', v,
				'value', v,
				'hasArrow', true
			)
		end
		result=nil
		added = nil
		if Manufac.db.profile.canmake then
			dewdrop:AddLine(
				'text', L["Can Make"],
				'value', "canmake",
				'hasArrow', true
			)
		end
		--[[
		--cannot CastSpell disenchant :(
		if skill==BS["Enchanting"] then
			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', BS["Disenchant"],
				'func', function() CastSpell(skills[BS["Disenchant"] ],"spell") end
			)
		end
		]]
		if arg6==nil then
			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', L["Options"],
				'value', L["Options"],
				'hasArrow', true
			)
			--and a refresh button
			dewdrop:AddLine(
				'text', L["Refresh"],
				'func', Manufac.Refresh
			)
			dewdrop:AddLine(
				'text', L["Close"],
				'func', function()
					dewdrop:Close()
					HideUIPanel(CraftFrame)
					HideUIPanel(TradeSkillFrame)
				end
			)
		end
		Manufac:Debug("-----FIRST LEVEL COMPLETE-----")
	elseif arg2==nil then
		Manufac:Debug("-----STARTING SECOND LEVEL-----")
		--ok, we're moused over a category. now lets make a list of recipes in that category.
		local tosort={}
		if skill == nil then Manufac:Debug("line 610") return end
		--sort the stuff!
		if arg1=="canmake" then
			local maxindex
			local haveone=false
			if skill == BS["Enchanting"] then
				maxindex = GetNumCrafts()
			else
				maxindex = GetNumTradeSkills()
			end
			for i=1,maxindex do
				local h=i
				local item = stitch:GetItemDataByIndex(skill, i)
				if type(item) == "table" and item.numcraftable and item.numcraftable > 0 then
					haveone=true
					if Manufac.db.profile.l2tooltips==true then
						dewdrop:AddLine(
							'text', (colors[item.difficulty]..item.name.."|r ["..item.numcraftable.."/"..item.numcraftablewbank.."]"),
							'value', h,
							'hasArrow', true,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', h
						)
					else
						dewdrop:AddLine(
							'text', (colors[item.difficulty]..item.name.."|r ["..item.numcraftable.."/"..item.numcraftablewbank.."]"),
							'value', h,
							'hasArrow', true,
							'tooltipFunc',nil
						)
					end
				end
			end
			if haveone==false then
				dewdrop:AddLine(
					'text', L["Cannot Create Anything"],
					'isTitle', true
				)
			end
			return
		end
		if Manufac.db.profile.sorttype == L["Alpha"] then
			for k,v in pairs (Manufac.db.profile.groups[skill]) do
				if v == arg1 then
					local info = stitch:GetItemDataByName(k)
					if not info then Manufac.db.profile.groups[skill][k] = nil Manufac:Debug("line656!",skill,k) return end
					table.insert(tosort, k)
				end
			end
		else
			local newname
			for k,v in pairs (Manufac.db.profile.groups[skill]) do
				if v == arg1 then
					local info = stitch:GetItemDataByName(k)
					if not info then Manufac.db.profile.groups[skill][k] = nil Manufac:Debug("line664!",skill,k) return end
					if info.difficulty=="optimal" then
						newname="1"..info.name
					elseif info.difficulty=="medium" then
						newname="2"..info.name
					elseif info.difficulty=="easy" then
						newname="3"..info.name
					else
						newname="4"..info.name
					end
					table.insert(tosort, newname)
				end
			end
		end
		table.sort(tosort)
		for k,v in pairs(tosort) do
			local skillname = Manufac:BreakOffNumber(tostring(v))
			local info = stitch:GetItemDataByName(skillname)
			Manufac:Debug(skillname)
			if not info then Manufac:Debug("line 681") return end
			if info.numcraftablewbank > 0 then
				if Manufac.db.profile.l2tooltips==true then
					dewdrop:AddLine(
						'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
						'value', info.index,
						'hasArrow', true,
						'tooltipFunc', Manufac.SetTooltip,
						'tooltipArg1', GameTooltip, 'tooltipArg2', info.index
					)
				else
					dewdrop:AddLine(
						'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
						'value', info.index,
						'hasArrow', true,
						'tooltipFunc',nil
					)
				end
			else
				if Manufac.db.profile.l2tooltips==true then
					dewdrop:AddLine(
						'text', (colors[info.difficulty]..info.name.."|r"),
						'value', info.index,
						'hasArrow', true,
						'tooltipFunc', Manufac.SetTooltip,
						'tooltipArg1', GameTooltip, 'tooltipArg2', info.index
					)
				else
					dewdrop:AddLine(
						'text', (colors[info.difficulty]..info.name.."|r"),
						'value', info.index,
						'hasArrow', true,
						'tooltipFunc',nil
					)
				end
			end
		end
		tosort=nil
		Manufac:Debug("-----SECOND LEVEL COMPLETE-----")
	elseif arg3==nil then
		Manufac:Debug("-----STARTING THIRD LEVEL-----")
		local info = stitch:GetItemDataByIndex(skill, arg1)
		if arg2=="canmake" then
			arg2=Manufac.db.profile.groups[skill][info.name]
		end
		local totalprice = 0
		local displaytotalprice = true
		if skill == BS["Enchanting"] then
			if Manufac.db.profile.l3tooltips==true then
				dewdrop:AddLine(
					'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
					'tooltipFunc', Manufac.SetTooltip,
					'tooltipArg1', GameTooltip, 'tooltipArg2', arg1,
					'func', Manufac.PrintSingleLink,
					'arg1', info.link,
					'arg2', info.link
				)
			else
				dewdrop:AddLine(
					'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
					'tooltipFunc', nil,
					'func', Manufac.PrintSingleLink,
					'arg1', info.link,
					'arg2', info.link
				)
			end
			dewdrop:AddLine()
			--cast this enchant
			dewdrop:AddLine(
				'text', "|cFF00FF00"..L["Cast"],
				'func', Manufac.ClickItemOne,
				'arg2', arg1,
				'arg1', arg1
			)
			dewdrop:AddLine()
			--do we need a rod? make it a line.
			if info.tools then
				dewdrop:AddLine(
					'text', info.tools,
					'notClickable', true
				)
			end
			--Note, if enchants ever get cooldowns, that goes here.
		else
			if Manufac.db.profile.l3tooltips==true then
				dewdrop:AddLine(
					'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
					'tooltipFunc', Manufac.SetTooltip,
					'tooltipArg1', GameTooltip, 'tooltipArg2', arg1,
					'func', Manufac.PrintSingleLink,
					'arg1', info.link,
					'arg2', info.link
				)
			else
				dewdrop:AddLine(
					'text', (colors[info.difficulty]..info.name.."|r ["..info.numcraftable.."/"..info.numcraftablewbank.."]"),
					'tooltipFunc', nil,
					'func', Manufac.PrintSingleLink,
					'arg1', info.link,
					'arg2', info.link
				)
			end
			if Manufac.db.profile.money==true then
				local sell, buy, buyout = Manufac:GetPriceData(info.link)
				if buyout and buyout > 0 then
					buyout = buyout*info.nummade
					dewdrop:AddLine(
						'text', ("Market Value: "..Manufac:GetMoneyText(buyout)),
						'isTitle', true
					)
				end
				if sell and sell > 0 then
					sell = sell*info.nummade
					dewdrop:AddLine(
						'text', ("Vendor Value: "..Manufac:GetMoneyText(sell)),
						'isTitle', true
					)
				end
			end
			dewdrop:AddLine()
			--cast one
			Manufac:Debug("more lines")
			dewdrop:AddLine(
				'text', "|cFF00FF00"..string.format(L["Create %s"],info.nummade),
				'func', Manufac.ClickItemOne,
				'arg1', info.index,
				'arg2', info.index
			)
			--create some
			dewdrop:AddLine(
				'text', "|cFF00FF00"..L["Create X"],
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxFunc', function(a,b) Manufac:ClickItemNum(a,b) end,
				'editBoxArg1', info.index
			)
			--create all
			dewdrop:AddLine(
				'text', "|cFF00FF00"..L["Create All"],
				'func', Manufac.ClickItemAll,
				'arg2', info.index,
				'arg1', info.index
			)
			dewdrop:AddLine()
			--list requirements (ie anvil)
			if info.tools then
				dewdrop:AddLine(
					'text', info.tools,
					'notClickable', true
				)
			end
			local cooldown=GetTradeSkillCooldown(info.index)
			if cooldown ~= nil and cooldown >0 then
				dewdrop:AddLine(
					'text', "|cFFFF3333"..SecondsToTime(cooldown).."|r",
					'notClickable', true
				)
			end
		end
		Manufac:Debug("STARTING REAGENTS")
		for i=1,info.numreagents do
			local h=i
			local price
			local inprof=false
			local reagentdata = stitch:GetItemDataByName(info[i]["name"])
			if reagentdata then
				inprof=reagentdata.index
			end
			if Manufac.db.profile.money==true then
				local sell, buy, buyout = Manufac:GetPriceData(info[i]["link"])
				if buy and buy > 0 and (((not buyout) or buyout == 0) or (not Auctioneer)) then
					price = buy*info[i]["needed"]
					totalprice = price + totalprice
					price = Manufac:GetMoneyText(price)
				elseif buyout and buyout > 0 then
					price = buyout*info[i]["needed"]
					totalprice = price + totalprice
					price = Manufac:GetMoneyText(price)
				else
					price = L["Unknown"]
					displaytotalprice = false
				end
			else
				price = ""
				displaytotalprice=false
			end
			if info[i].vendor then
				if tonumber(inprof) then
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(vendor)"].." "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h,
							'hasArrow', true,
							'value', info[i].name
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(vendor)"].." "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil,
							'hasArrow', true,
							'value', info[i].name
						)
					end
				else
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(vendor)"].." "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(vendor)"].." "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil
						)
					end
				end
			elseif info[i].numwbank > info[i].num then
				if tonumber(inprof) then
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..") "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h,
							'hasArrow', true,
							'value', info[i].name
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..") "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil,
							'hasArrow', true,
							'value', info[i].name
						)
					end
				else
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..") "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..") "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil
						)
					end
				end
			else
				if tonumber(inprof) then
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h,
							'hasArrow', true,
							'value', info[i].name
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil,
							'hasArrow', true,
							'value', info[i].name
						)
					end
				else
					if Manufac.db.profile.l3tooltips==true then
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', Manufac.SetTooltip,
							'tooltipArg1', GameTooltip, 'tooltipArg2', arg1, 'tooltipArg3', h
						)
					else
						dewdrop:AddLine(
							'text', ("|cFF6666FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..price),
							'func', Manufac.PrintSingleLink,
							'arg1', info[i].link,
							'arg2', info[i].link,
							'tooltipFunc', nil
						)
					end
				end
			end

		end
		Manufac:Debug("DONEW/ REAGENTS")
		if displaytotalprice then
			totalprice = Manufac:GetMoneyText(totalprice)
			dewdrop:AddLine(
				'text', ("Mats Cost: "..totalprice),
				'isTitle', true
			)
		end
		Manufac:Debug("LINES1")
		dewdrop:AddLine()
		--linking lines

		dewdrop:AddLine(
			'text', ("|cFFFFFF00"..L["LinkResult"].."|r"),
			'func', Manufac.PrintSingleLink,
			'arg1', info.link,
			'arg2', info.link
		)
		Manufac:Debug("LINES2")
		dewdrop:AddLine(
			'text', ("|cFFFFFF00"..L["LinkRecipe!"].."|r"),
			'func', Manufac.PrintMultiLink,
			'arg2', info,
			'arg1', info
		)
		Manufac:Debug("LINES3")
		dewdrop:AddLine()
		--category setup
		dewdrop:AddLine(
			'text', L["Grouping"],
			'hasArrow', true,
			'hasEditBox', true,
			'editBoxFunc', function(item, group,text)
				Manufac:MakeDBChange(item,group,text)
			end,
			'editBoxArg1', arg1,
			'editBoxArg2', arg2
		)
		Manufac:Debug("-----THIRD LEVEL COMPLETE-----")
	elseif arg4==nil then
		Manufac:Debug("-----STARTING FOURTH LEVEL-----")
		--Note, this will explode if an enchant ever uses a mat created by enchanting, ie a mana oil
		local info = stitch:GetItemDataByName(arg1)
		local group = Manufac.db.profile.groups[skill][arg1]
		dewdrop:AddLine(
			'text', "|cFF00FF00"..string.format(L["Create %s"],info.nummade),
			'func', Manufac.ClickItemOne,
			'arg1', info.index,
			'arg2', info.index
		)
		--create some
		dewdrop:AddLine(
			'text', "|cFF00FF00"..L["Create X"],
			'hasArrow', true,
			'hasEditBox', true,
			'editBoxFunc', function(a,b) Manufac:ClickItemNum(a,b) end,
			'editBoxArg1', info.index
		)
		--create all
		dewdrop:AddLine(
			'text', "|cFF00FF00"..L["Create All"],
			'func', Manufac.ClickItemAll,
			'arg2', info.index,
			'arg1', info.index
		)
		dewdrop:AddLine()
		for i=1,info.numreagents do
			local h=i
			if info[i].numwbank > info[i].num then
				if Manufac.db.profile.l3tooltips==true then
					dewdrop:AddLine(
						'text', ("|cFF4444FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..")"),
						'func', Manufac.PrintSingleLink,
						'arg1', info[i].link,
						'arg2', info[i].link,
						'tooltipFunc', Manufac.SetTooltip,
						'tooltipArg1', GameTooltip, 'tooltipArg2', info.index, 'tooltipArg3', h
					)
				else
					dewdrop:AddLine(
						'text', ("|cFF4444FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."] "..L["(bank: "]..info[i].numwbank - info[i].num..")"),
						'func', Manufac.PrintSingleLink,
						'arg1', info[i].link,
						'arg2', info[i].link,
						'tooltipFunc', nil
					)
				end
			else
				if Manufac.db.profile.l3tooltips==true then
					dewdrop:AddLine(
						'text', ("|cFF4444FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."]"),
						'func', Manufac.PrintSingleLink,
						'arg1', info[i].link,
						'arg2', info[i].link,
						'tooltipFunc', Manufac.SetTooltip,
						'tooltipArg1', GameTooltip, 'tooltipArg2', info.index, 'tooltipArg3', h
					)
				else
					dewdrop:AddLine(
						'text', ("|cFF4444FF"..info[i].name.."|r ["..info[i].num.."/"..info[i].needed.."]"),
						'func', Manufac.PrintSingleLink,
						'arg1', info[i].link,
						'arg2', info[i].link,
						'tooltipFunc', nil
					)
				end
			end
		end

	end
	timer = GetTime()-timer
	Manufac:Debug(timer)
end
function Manufac:GetSkillLevel(arg1)
	--if arg1 == L["FirstAid"] then arg1 = L["First Aid"] end
	--if arg1 == L["Smelting"] then arg1 = L["Mining"] end
	local skilltext = ""
	for i=1,GetNumSkillLines() do
		local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription = GetSkillLineInfo(i)
		if skillName and skillName == arg1 then
			skilltext = "|cFF7777FF ("..(skillRank+numTempPoints+skillModifier).."/"..skillMaxRank..")|r"
		end
	end
	return skilltext
end
function Manufac:GetPriceData(link)
	if link==nil then
		return 0,0,0
	end
	if Manufac.db.profile.money==false then return nil, nil, nil end
	if KC_Items then
		local resultinfo = KC_Items.common:GetItemInfo(KC_Items.common:GetCode(link))
		local sSeen, sAvgStack, sMin, sBidcount, sBid, sBuycount, sBuy = KC_Auction:GetItemData(string.gsub(resultinfo["hyperlink"], "item:", ""))
		return resultinfo["sell"], resultinfo["buy"],sBuy
	elseif Auctioneer and Informant and EnhTooltip then --run away screaming
		Manufac:Debug("in auctioneer.")
		local itemid = (gsub(link, "^.*item:(%d+):.*$", "%1"))
		local itemKey = itemid..":0:0"
		local itemTotals = Auctioneer.Core.GetAuctionPriceItem(itemKey, Auctioneer.Util.GetAuctionKey())
		local aCount,minCount,minPrice,bidCount,bidPrice,buyCount,buyPrice = Auctioneer.Core.GetAuctionPrices(itemTotals.data);
		local avgBuy=0
		if itemTotals and (buyCount > 0) then
			avgBuy = math.floor(buyPrice / buyCount)
		end
		local venddata = Informant.GetItem(tonumber(itemid))
		if venddata then
			return venddata['sell'], venddata['buy'], avgBuy
		else
			return 0, 0, avgBuy
		end
	end
end
function Manufac:GetMoneyText(num)
	if KC_Items then
		return KC_Common:CashTextLetters(num)
	elseif EnhTooltip then
		return EnhTooltip.GetTextGSC(num)
	else
		return ""
	end
end
function Manufac:GetSkillName(arg)
	if arg then
		if arg == BS["Enchanting"] then
			if CraftFrameTitleText and CraftFrameTitleText:GetText()==BS["Enchanting"] then
				skill=BS["Enchanting"]
			end
		elseif TradeSkillFrameTitleText then
			skill=TradeSkillFrameTitleText:GetText()
			--if skill==L["First Aid"] then
			--	skill = L["FirstAid"]
			--elseif skill==L["Mining"] then
			--	skill = L["Smelting"]
			--end
		else skill = nil end
	else
		Manufac:Debug("recentopen",recentopen)
		if recentopen=="Trade" then
			skill=TradeSkillFrameTitleText:GetText()
			--if skill==L["First Aid"] then
			--	skill = L["FirstAid"]
			--elseif skill==L["Mining"] then
			--	skill = L["Smelting"]
			--end
		elseif recentopen=="Craft" then
			skill = CraftFrameTitleText:GetText()
			if skill ~= BS["Enchanting"] then skill = nil end
		else
			skill=nil
		end
	end
end
function Manufac:BreakOffNumber(v)
	if tonumber(string.sub(v,1,1)) then
		return string.sub(v,2)
	else
		return v
	end
end
function Manufac:ScanTrade(profession)
	Manufac:Debug("profession",profession)
	professionpoints[profession] = {}
	if Manufac.db.profile.groups[profession]== nil then
		Manufac.db.profile.groups[profession] = {}
	end
	if profession == BS["Enchanting"] then
		for i=1,GetNumCrafts() do
			local skillname, _, skilltype = GetCraftInfo(i)
			if skilltype~="header" and skillname then
				local link = GetCraftItemLink(i)
				local id = Manufac:GetIDFromLink(link)
				if (not(Manufac.db.profile.groups[profession][skillname]) or (Manufac.db.profile.groups[profession][skillname] == L["Unsorted"])) then
					if loadeddefaults[profession] and loadeddefaults[profession][id] then
						Manufac.db.profile.groups[profession][skillname] = loadeddefaults[profession][id]
					else
						Manufac.db.profile.groups[profession][skillname] = L["Unsorted"]
					end
				end
			end
		end
	else
		for i=1,GetNumTradeSkills() do
			local skillname, skilltype = GetTradeSkillInfo(i)
			if skilltype~="header" and skillname then
				local link = GetTradeSkillItemLink(i)
				local id = Manufac:GetIDFromLink(link)
				if (not(Manufac.db.profile.groups[profession][skillname]) or (Manufac.db.profile.groups[profession][skillname] == L["Unsorted"])) then
					if loadeddefaults[profession] and loadeddefaults[profession][id] then
						Manufac.db.profile.groups[profession][skillname] = loadeddefaults[profession][id]
					else
						Manufac.db.profile.groups[profession][skillname] = L["Unsorted"]
					end
				end
			end
		end
	end
	professionpoints[profession] = Manufac:GetSkillLevel(profession)
end
function Manufac:MakeDBChange(index, oldgroup, text)
	if text ~= L["Options"] and text ~= "level" then
		local name
		local id
		if skill==BS["Enchanting"] then
			name=GetCraftInfo(index)
		else
			name=GetTradeSkillInfo(index)
		end
		Manufac.db.profile.groups[skill][name] = text

		dewdrop:Refresh(1)
		dewdrop:Refresh(2)
	end
end
function Manufac:GetIDFromLink(link)
	if (not link) then return nil end
	local id = string.match(link, "item:(%d+)")
	if id and tonumber(id) > 0 then
		Manufac:Debug(id)
		return id
	else
		local id = string.match(link, "enchant:(%d+)")
		Manufac:Debug(id)
		return id
	end
end
function Manufac:ClickItemOne(item)
	Manufac:Debug(item, " clicked")
	if skill == BS["Enchanting"] then
		DoCraft(item)
	else
		DoTradeSkill(item)
	end
end
function Manufac:ClickItemAll(item)
	_, _, numavailable = GetTradeSkillInfo(item)
	if numavailable then
		DoTradeSkill(item, numavailable)
	end
end
function Manufac:ClickItemNum(item, num)
	DoTradeSkill(item, num)
end
function Manufac:SetTooltip(arg1, arg2)
	--Manufac:Debug("setting tooltip:",arg1)
	GameTooltip_SetDefaultAnchor(GameTooltip, this)
	if skill == BS["Enchanting"] then
		if arg2 and GetCraftInfo(arg1) then
			GameTooltip:SetCraftItem(arg1, arg2)
		else
			GameTooltip:SetCraftSpell(arg1)
		end
	else
		if arg2 and GetTradeSkillInfo(arg1) then
			GameTooltip:SetTradeSkillItem(arg1, arg2)
		else
			GameTooltip:SetTradeSkillItem(arg1)
		end
	end
end
function Manufac:PrintMultiLink(itemtable)
	local text = ""
	if not itemtable then return end
	if itemtable.nummade > 1 then
		text = itemtable.nummade.."x"
	end
	if itemtable.link==nil then
		return
	end
	text = text..itemtable.link..L[" requires: "]
	for i=1,itemtable.numreagents do
		if i > 1 then text = text .. ", " end
		text = text..itemtable[i]["needed"].."x "..itemtable[i]["name"]
	end
	if ( ChatFrameEditBox:IsVisible() ) then
		ChatFrameEditBox:Insert(text)
	else
		ChatFrame_OpenChat(text)
	end
end
function Manufac:PrintSingleLink(link)
	if ( ChatFrameEditBox:IsVisible() ) then
		ChatFrameEditBox:Insert(link)
	else
		ChatFrame_OpenChat(link)
	end
end
function Manufac:ModifyTradeSkill()
	if TradeSkillFrame and TradeSkillFrame:GetAlpha()>0 then
		TradeSkillFrame:EnableMouse(false)
		TradeSkillFrame:SetAlpha(0)
		TradeSkillCancelButton:EnableMouse(false)
		TradeSkillCreateButton:EnableMouse(false)
		TradeSkillIncrementButton:EnableMouse(false)
		TradeSkillInputBox:EnableMouse(false)
		TradeSkillDecrementButton:EnableMouse(false)
		TradeSkillCreateAllButton:EnableMouse(false)
		TradeSkillReagent1:EnableMouse(false)
		TradeSkillReagent2:EnableMouse(false)
		TradeSkillReagent3:EnableMouse(false)
		TradeSkillReagent4:EnableMouse(false)
		TradeSkillReagent5:EnableMouse(false)
		TradeSkillReagent6:EnableMouse(false)
		TradeSkillReagent7:EnableMouse(false)
		TradeSkillReagent8:EnableMouse(false)
		TradeSkillSkillIcon:EnableMouse(false)
		TradeSkillSkill1:EnableMouse(false)
		TradeSkillSkill2:EnableMouse(false)
		TradeSkillSkill3:EnableMouse(false)
		TradeSkillSkill4:EnableMouse(false)
		TradeSkillSkill5:EnableMouse(false)
		TradeSkillSkill6:EnableMouse(false)
		TradeSkillSkill7:EnableMouse(false)
		TradeSkillSkill8:EnableMouse(false)
		TradeSkillListScrollFrameScrollBar:EnableMouse(false)
		TradeSkillListScrollFrameScrollBarScrollUpButton:EnableMouse(false)
		TradeSkillListScrollFrameScrollBarScrollDownButton:EnableMouse(false)
		TradeSkillCollapseAllButton:EnableMouse(false)
		TradeSkillSubClassDropDownButton:EnableMouse(false)
		TradeSkillInvSlotDropDownButton:EnableMouse(false)
		TradeSkillRankFrameBorder:EnableMouse(false)
		TradeSkillFrameCloseButton:EnableMouse(false)
		TradeSkillFrame:UnregisterEvent("TRADESKILL_UPDATE")
	end
end
function Manufac:UnModifyTradeSkill()
	if TradeSkillFrame~=nil then
		--Manufac:Debug("UNMODING TRADESKILL")
		TradeSkillFrame:EnableMouse(true)
		TradeSkillFrame:SetAlpha(1)
		TradeSkillCancelButton:EnableMouse(true)
		TradeSkillCreateButton:EnableMouse(true)
		TradeSkillIncrementButton:EnableMouse(true)
		TradeSkillInputBox:EnableMouse(true)
		TradeSkillDecrementButton:EnableMouse(true)
		TradeSkillCreateAllButton:EnableMouse(true)
		TradeSkillReagent1:EnableMouse(true)
		TradeSkillReagent2:EnableMouse(true)
		TradeSkillReagent3:EnableMouse(true)
		TradeSkillReagent4:EnableMouse(true)
		TradeSkillReagent5:EnableMouse(true)
		TradeSkillReagent6:EnableMouse(true)
		TradeSkillReagent7:EnableMouse(true)
		TradeSkillReagent8:EnableMouse(true)
		TradeSkillSkillIcon:EnableMouse(true)
		TradeSkillSkill1:EnableMouse(true)
		TradeSkillSkill2:EnableMouse(true)
		TradeSkillSkill3:EnableMouse(true)
		TradeSkillSkill4:EnableMouse(true)
		TradeSkillSkill5:EnableMouse(true)
		TradeSkillSkill6:EnableMouse(true)
		TradeSkillSkill7:EnableMouse(true)
		TradeSkillSkill8:EnableMouse(true)
		TradeSkillListScrollFrameScrollBar:EnableMouse(true)
		TradeSkillListScrollFrameScrollBarScrollUpButton:EnableMouse(true)
		TradeSkillListScrollFrameScrollBarScrollDownButton:EnableMouse(true)
		TradeSkillCollapseAllButton:EnableMouse(true)
		TradeSkillSubClassDropDownButton:EnableMouse(true)
		TradeSkillInvSlotDropDownButton:EnableMouse(true)
		TradeSkillRankFrameBorder:EnableMouse(true)
		TradeSkillFrameCloseButton:EnableMouse(true)
		TradeSkillFrame:RegisterEvent("TRADESKILL_UPDATE")
	end
end
function Manufac:ModifyCraft()
	if CraftFrame and CraftFrame:GetAlpha()>0 then
		CraftFrame:EnableMouse(false)
		CraftFrame:SetAlpha(0)
		CraftCancelButton:EnableMouse(false)
		CraftCreateButton:EnableMouse(false)
		CraftReagent1:EnableMouse(false)
		CraftReagent2:EnableMouse(false)
		CraftReagent3:EnableMouse(false)
		CraftReagent4:EnableMouse(false)
		CraftReagent5:EnableMouse(false)
		CraftReagent6:EnableMouse(false)
		CraftReagent7:EnableMouse(false)
		CraftReagent8:EnableMouse(false)
		CraftIcon:EnableMouse(false)
		Craft1:EnableMouse(false)
		Craft2:EnableMouse(false)
		Craft3:EnableMouse(false)
		Craft4:EnableMouse(false)
		Craft5:EnableMouse(false)
		Craft6:EnableMouse(false)
		Craft7:EnableMouse(false)
		Craft8:EnableMouse(false)
		CraftListScrollFrameScrollBar:EnableMouse(false)
		CraftListScrollFrameScrollBarScrollUpButton:EnableMouse(false)
		CraftListScrollFrameScrollBarScrollDownButton:EnableMouse(false)
		CraftRankFrameBorder:EnableMouse(false)
		CraftFrameCloseButton:EnableMouse(false)
		CraftFrame:UnregisterEvent("CRAFT_UPDATE")
	end
end
function Manufac:UnModifyCraft()
	if CraftFrame~= nil then
		CraftFrame:EnableMouse(true)
		CraftFrame:SetAlpha(1)
		CraftCancelButton:EnableMouse(true)
		CraftCreateButton:EnableMouse(true)
		CraftReagent1:EnableMouse(true)
		CraftReagent2:EnableMouse(true)
		CraftReagent3:EnableMouse(true)
		CraftReagent4:EnableMouse(true)
		CraftReagent5:EnableMouse(true)
		CraftReagent6:EnableMouse(true)
		CraftReagent7:EnableMouse(true)
		CraftReagent8:EnableMouse(true)
		CraftIcon:EnableMouse(true)
		Craft1:EnableMouse(true)
		Craft2:EnableMouse(true)
		Craft3:EnableMouse(true)
		Craft4:EnableMouse(true)
		Craft5:EnableMouse(true)
		Craft6:EnableMouse(true)
		Craft7:EnableMouse(true)
		Craft8:EnableMouse(true)
		CraftListScrollFrameScrollBar:EnableMouse(true)
		CraftListScrollFrameScrollBarScrollUpButton:EnableMouse(true)
		CraftListScrollFrameScrollBarScrollDownButton:EnableMouse(true)
		CraftRankFrameBorder:EnableMouse(true)
		CraftFrameCloseButton:EnableMouse(true)
		CraftFrame:RegisterEvent("CRAFT_UPDATE")
	end
end
function Manufac:Refresh()
	dewdrop:Refresh(1)
end
function Manufac:ScanActionButtons()
	local _, _, _, numSpells = GetSpellTabInfo(1)
	skills = {}
	for h=1,numSpells do
		local spellName= GetSpellName(h, BOOKTYPE_SPELL)
		if spellName == BS["Alchemy"] then
			skills[BS["Alchemy"]] = h
			Manufac:CheckDefaults(BS["Alchemy"])
		end
		if spellName == BS["Engineering"] then
			skills[BS["Engineering"]] = h
			Manufac:CheckDefaults(BS["Engineering"])
		end
		if spellName == BS["Enchanting"] then
			skills[BS["Enchanting"]] = h
			Manufac:CheckDefaults(BS["Enchanting"])
		end
		if spellName == BS["Leatherworking"] then
			skills[BS["Leatherworking"]] = h
			Manufac:CheckDefaults(BS["Leatherworking"])
		end
		if spellName == BS["Tailoring"] then
			skills[BS["Tailoring"]] = h
			Manufac:CheckDefaults(BS["Tailoring"])
		end
		if spellName == BS["Blacksmithing"] then
			skills[BS["Blacksmithing"]] = h
			Manufac:CheckDefaults(BS["Blacksmithing"])
		end
		if spellName == BS["First Aid"] then
			--skills[L["FirstAid"]] = h
			skills[BS["First Aid"]] = h
			Manufac:CheckDefaults(BS["First Aid"])
		end
		if spellName == BS["Cooking"] then
			skills[BS["Cooking"]] = h
			Manufac:CheckDefaults(BS["Cooking"])
		end
		if spellName == BS["Jewelcrafting"] then
			skills[BS["Jewelcrafting"]] = h
			Manufac:CheckDefaults(BS["Jewelcrafting"])
		end
		if spellName == BS["Smelting"] then
			skills[BS["Mining"]] = h
			Manufac:CheckDefaults(BS["Mining"])
		end
		if spellName == BS["Poisons"] then
			skills[BS["Poisons"]] = h
			Manufac:CheckDefaults(BS["Poisons"])
		end
		if spellName == BS["Disenchant"] then
			skills[BS["Disenchant"]] = h
		end
	end
	if Manufac_Defaults then
		Manufac_Defaults = nil
		Manufac:Debug("Defaults Reclaimed")
	end
end
function Manufac:ListUnsortedRecipes()
	for k,v in pairs(Manufac.db.profile.groups) do
		for a,b in pairs(v) do
			if a and b and b == L["Unsorted"] and stitch:GetItemDataByName(a) then
				Manufac:Print(a,Manufac:GetIDFromLink(stitch:GetItemDataByName(a).link))
			end
		end
	end
end
function Manufac:SetFubarPluginState()
	if ManufacFu then
		if Manufac.db.profile.usefubar==true then
			ManufacFu:Show()
		else
			ManufacFu:Hide()
		end
	end
end

--put resultant item's details in the menu itself?
--option for multi-line recipe link output with links
--if a reagent hasarrow, then include the number that can be crafted with mats on hand?

--sorting for "canmake"
--when click 'major mana potion' in menu, instead of a submenu, open a tablet with the info.
